__revision__  = "$Revision: 845 $"
__date__      = "$Date: 2007-01-08 15:54:59 +0100 (Mon, 08 Jan 2007) $"
__copyright__ = "Copyright (c) 2004 by banality agd"
__author__    = "Simon Pamies"
__full_info__ = "$Id: ProductsInstallHelper.py 845 2007-01-08 14:54:59Z spamies $"
__licence__   = "see doc/licence.txt"

from StringIO import StringIO
from Products.Archetypes.public import listTypes
from Products.Archetypes.Extensions.utils import installTypes, install_subskin
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.Expression import Expression
from Products.CMFCore.ActionInformation import ActionInformation
from Products.CMFCore.permissions import setDefaultRoles
from Products.GenericSetup.interfaces import IBody
from Products.GenericSetup.interfaces import ISetupEnviron
from zope.component import queryMultiAdapter
from zope.interface import implements
from types import ListType, TupleType, StringTypes
from Acquisition import aq_base
from AccessControl.SpecialUsers import system as UnrestrictedUser 
from AccessControl.SecurityManagement import newSecurityManager, noSecurityManager
import os, sys

import logging
log = logging.getLogger('AnthillCMS')

HAS_LXML = True
try:
    from lxml import etree
    log.info('Starting up anthill utilities with full profile import caps.')
except ImportError:
    HAS_LXML = False

try:
    blubb = True
except NameError:
    True  = 1
    False = 0

class SetupEnviron(object):
    """
    Context - copied from GenericSetup
    """

    implements(ISetupEnviron)

    def __init__(self, site=None, profile_path='', purge=True):
        self.site = site
        self._profile_path = profile_path
        self._dopurge = purge

    def getSite(self):
        return self.site

    def getEncoding(self):
        return 'utf-8'

    def getLogger(self, name):
        return logging.getLogger('GenericSetup.%s' % name)

    def shouldPurge(self):
        """ If this returns True the all data is purged prior to import """
        return self._dopurge

    def writeDataFile(self, filename, body, mimetype):
        try: os.mkdir(os.path.dirname(filename)) # create export directories
        except: pass

        f = open(filename, 'w')
        f.write(body)
        f.close()

    def readDataFile( self, filename, subdir=None ):
        """ See IImportContext or GenericSetup/context.py
        """
        if subdir is None:
            full_path = os.path.join( self._profile_path, filename )
        else:
            full_path = os.path.join( self._profile_path, subdir, filename )

        if not os.path.exists( full_path ):
            return None

        f = open(full_path, 'rb')
        result = f.read()
        f.close()
        return result

class ProductsInstallHelper:

    def __init__(self, productname, context):
        self._import_dir = ''
        self._productname = productname
        self._context = context
        self._root = self.getPortalObject() 

    # ***********************************
    # UTILITY FUNCTIONS
    # ***********************************
    
    def cleanUp(self):
        self.msg('Finished Installation of %s - Cleaning up...' % self._productname)

    def msg(self, message):
        log.info(message)

    def error(self, message):
        log.error(message)

    def exception(self, message):
        raise Exception, message

    def info(self, message):
        self.msg(message)

    def getContext(self):
        return self._context

    def getProductName(self):
        return self._productname

    def getPortalObject(self):
        return getToolByName(self.getContext(), 'portal_url').getPortalObject()

    getPortalRoot = getPortalObject

    def getPortalProperties(self):
        return getToolByName(self.getContext(), 'portal_properties')
    
    def getHomeDir(self):
        """
        Returns the home directory of
        the Product given
        """
        
        return self._context.getPhysicalRoot().Control_Panel.Products._getOb(self._productname).home

    def loginAsManager(self):
        newSecurityManager(None, UnrestrictedUser)

    def logout(self):
        noSecurityManager()

    # ***********************************
    # TYPES AND SKINS
    # ***********************************
    
    def installSkins(self, globals=globals()):
        self.msg('Installing skin for %s' % self._productname)
        install_subskin(self._context, StringIO(), globals)

    def installTypes(self):
        types = listTypes(self._productname)
        self.msg('Installing types')

        # helper message
        if not types:
            self.info('No Types found! Have you forgot to register some AT Class? (registerType(-classname-))')
            exit
        
        installTypes(self.getContext(), StringIO(), types, self._productname)

    def installTypesSkin(self, globals):
        self.installSkins(globals)
        self.installTypes()
    
    def installTool(self, portalname, title):
        """ Install tool from Extensions/Install

                inst.installTool(config.TOOL_NAME, 'Antz Tool')
                # Antz Tool is the meta_type of the tool!

            You have to init tool in __init__.py:

                from AntzCave import AntzTool
                tools = (AntzTool,)
                
                utils.ToolInit(meta_type="Antz Tool",
                               tools=tools,
                               icon='tool.jpg',
                               ).initialize(context)

        """
        tt = getToolByName(self.getPortalObject(), portalname, None)
        if tt is None: # not installed yet
            addprod = self.getPortalObject().manage_addProduct[self.getProductName()]
            if not hasattr(addprod, 'manage_addTool'):
                self.error('You do not seem to have initialized the tool via CMFCore.utils.ToolInit in your __init__.py!')

            addTool = addprod.manage_addTool
            addTool(title)

            tt = getToolByName(self.getPortalObject(), portalname)
            at = getToolByName(self.getPortalObject(), 'portal_actions')
            at.addActionProvider(portalname)
            self.msg('Installed tool %s' % title)
        else:
            self.msg('Tool %s already installed!' % title)
        
    # ***********************************
    # DEPENDENCIES
    # ***********************************

    def installDependencies(self, dependencies):

        pqi = getToolByName(self.getContext(), 'portal_quickinstaller')
        for depend in dependencies:
            self.installProduct(depend)

    def installProduct(self, name):

        pqi = getToolByName(self.getContext(), 'portal_quickinstaller')
        if pqi.isProductInstalled(name):
            self.msg('%s is already installed - ignoring it' % name)
            return

        self.msg('Trying to install %s...' % name)

        try:
            pqi.installProduct(name)
            self.msg('Installed %s' % name)
            
        except Exception, e:
            self.error('Could not install %s (%s)' % (name, str(e)))

    def uninstallProduct(self, products):

        if type(products) in StringTypes:
            products = [products, ]

        pqi = getToolByName(self.getContext(), 'portal_quickinstaller')
        for pd in products:
            if not pqi.isProductInstalled(pd):
                self.msg('%s not installed - skipping uninstall request' % pd)
                continue

            else:
                try:
                    pqi.uninstallProducts([pd, ])
                    self.msg('Removed product %s' % pd)
                except Exception, e:
                    self.error('Could not uninstall %s (%s)' % (pd, str(e)))

    def checkDependencies(self, dependencies):
        """ Checks dependencies and raise Exception
        if product is not installed that should be """

        pqi = getToolByName(self.getContext(), 'portal_quickinstaller')
        for depend in dependencies:
            if not pqi.isProductInstalled(depend):
                raise Exception, 'Product %s not installed. MUST be installed before installing %s' % (depend, self._productname)
    
    # ***********************************
    # FTI (TYPES) HANDLING
    # ***********************************
    
    def getFTIForType(self, name):
        """ Returns the fti for the given named type """

        pti = getToolByName(self.getContext(), 'portal_types')
        return pti.getTypeInfo(name)

    def addAliasForType(self, name, aliasfrom, aliasto):
        """ Sets alias for given type """

        fti = self.getFTIForType(name)
        aliases = fti.getMethodAliases()
        aliases.update({aliasfrom:aliasto})
        fti.setMethodAliases(aliases)
        self.msg('Added alias %s:%s for type %s' % (aliasfrom, aliasto, name))
   
    def disableGlobalTypeIsAddable(self, name, disable=True):
        """ Disables the implicit addable feature of the given type.
        That leads to the fact that the type isn't anymore addable through
        types that do not explicitely allow addition of that type """

        fti = self.getFTIForType(name)

        if not fti:
            raise Exception, '%s seems not to be a valid type! \
                    Please control your Install.py and double check disableGlobalTypeIsAddable calls. \
                    It can also happen that you have to call this method after installTypesAndSkins.' % name
        
        if disable:
            self.msg('Disallowed implicit creation of %s' % name)
            fti._setPropValue('global_allow', 0)
        else: fti._setPropValue('global_allow', 1)
  
    def disableGlobalTypesAreAddable(self, namelist, disable=True):
        """ Disables a list of types """

        for item in namelist:
            self.disableGlobalTypeIsAddable(item, disable)

    def addTypeToPortalFactory(self, typename):
        """ Adds portaltype to portal_factory so that
        temporary objects are created """

        fct = getToolByName(self.getContext(), 'portal_factory')
        fct._factory_types[typename] = 1
        fct._p_changed = 1
        self.msg('Added %s to portal_factory' % typename)

    # ***********************************
    # ACTION HANDLING
    # ***********************************

    def getActionsForType(self, type):
        """ Returns all actions defined for a specific type """

        tyt = getToolByName(self.getPortalObject(), 'portal_types')
        typeinfo = tyt.getTypeInfoFor(type)

        if typeinfo is not None:
            return typeinfo._cloneActions()
        
        return None

    def getActionsByProperties(self, toolname='portal_actions', **kw):
        """ Returns a list of actions from portal_actions matching kw """

        result = []
        pa = getToolByName(self.getPortalObject(), toolname)
        for action in pa._actions:
            count = 0
            for k,v in kw.items():
                if getattr(action, k, None) == kw[k]:
                    count += 1

            if count == len(kw.keys()):
                result.append(action)

        return result

    def saveBackModifiedAction(self, action, toolname='portal_actions'):
        """ Puts the given action back to portal_actions """

        pa = getToolByName(self.getPortalObject(), toolname)

        result = ()
        for act in pa._actions:
            if getattr(act, 'id') == getattr(action, 'id') and \
                    getattr(act, 'category') == getattr(action, 'category'):
                result += (action, )
            else: result += (act, )

        pa._actions = result
        pa._p_changed = 1

    def setActionCondition(self, action, conditionstring):
        """ Sets the condition for the given expression """

        action.condition = Expression(conditionstring)
        self.msg('Changed condition of action %s to %s' % (action.id, conditionstring))

    def setActionForType(self, type, actionid, **kw):
        """ Sets a field of the given action - dealing here with actions for types 
        Attributes must reside in **kw """

        tyt = getToolByName(self.getPortalObject(), 'portal_types')
        typeinfo = tyt.getTypeInfo(type)

        if typeinfo is not None:
            typeobj = getattr(tyt, typeinfo.getId())
            actions = typeinfo._cloneActions()
            
            for action in actions:
                if action.id == actionid:
                    for key in kw.keys():
                        self.msg('Changing attribute %s of action %s from type %s to %s' % (str(key), action.id, type, kw[key]))
                        setattr(action, key, kw[key])

            # saving changed actions
            typeobj._actions = tuple(actions)
   
    def createCompleteActionFor(self, object, id, title, 
            action, condition, permission, category, visible=1):
        """ Adds specified action to the object """

        if hasattr(object, 'addAction'):
            if object.getActionObject('%s/%s' % (category, id)) is None:
                object.addAction(id, title, 
                                 action, condition, permission, category, visible)
                self.msg('Added action %s to %s' % (title, object.getId()))

            else: self.info('Action %s on %s already installed' % (title, object.getId()))
        else: self.error('Could not add action %s to %s' % (title, object.getId()))
   
    # ***********************************
    # PERMISSIONS AND ROLES
    # ***********************************

    def ensurePermissionsToRole(self, name, permissions_list):
        pr = self.getPortalObject()
        self.ensurePermissionsToRoleFor(pr, name, permissions_list)

    def ensurePermissionsToRoleFor(self, object, name, permissions_list):
        object.manage_role(name, permissions_list)
        object._p_changed = 1
        [setDefaultRoles(name, perm) for perm in permissions_list]
        self.msg('Registered specific permissions %s for role %s' % (','.join(permissions_list), name))

    def createPortalObjectRoles(self, roles):
        pr = self.getPortalObject()
        if type(roles) != type([]):
            roles = [roles]
            
        data = list(pr.__ac_roles__)
        for role in roles:
            if role not in data:
                data.append(role)
                
        pr.__ac_roles__ = tuple(data)
        self.msg('Added new roles: %s' % str(roles))

    def roleIsInstalled(self, role):
        pr = self.getPortalObject()
        return role in list(pr.__ac_roles__)

    def testForAccessOf(self, directory):
        fileobject = StringIO('tescht')
        try:
            filename = 'tescht_file_importer'
            here = self.getHomeDir()
            myfile = open(os.path.join(here, directory, filename), 'wb')

            fileobject.read()
            size = fileobject.tell()
            if size == 0:
                size = -1
            
            fileobject.seek(0)
            inpdata = fileobject.read(size)
            myfile.write(inpdata)
            myfile.close()
        
            self.msg('Directory %s seems to be clean!' % os.path.join(here, directory))
            os.unlink(os.path.join(here, directory, filename))

        except:
            fileobject.close()
            raise Exception, 'ATTENTION: Please adjust the rights of %s!!' % os.path.join(here, directory)

    def importXMLSecurityTo(self, object, filename):
        """ Imports the xml security to the given object """

        if hasattr(object, 'manage_importSecurity'):
            product_home = self.getHomeDir()
            import_dir = os.path.join(INSTANCE_HOME, 'import')
            xmlfile = open(os.path.join(product_home, 'import', filename), 'rb')

            object.manage_importSecurity(filename=xmlfile)
            self.msg('Imported XML Security from %s' % filename)

        else:
            self.error('Could not find xml import method! Security NOT imported!')

    # ***********************************
    # ZEXP HANDLING
    # ***********************************

    def importWorkflow(self, workflow_file):
        """
        Imports a new workflow from ZEXP
        file into the workflow tool. That
        file MUST reside in the import directory
        of the Product!
        """

        wf = getToolByName(self.getPortalObject(), 'portal_workflow')        
        self.importContentTo(wf, [workflow_file])
        self.msg('Imported new workflow: %s' % workflow_file)

    def importContentTo(self, object, filename_list, copy_to_import_dir=1):
        """
        Imports the given objects into the given object
        """

        self.msg('Importing %s' % filename_list)
        if copy_to_import_dir:
            self._beforeProductFileImport(self._context, self._productname, filename_list)

        for file in filename_list:
            try:
                object.manage_importObject(file)
                self.msg('Imported %s to %s' % (str(file), str(object))) 
            except Exception, e:
                self.error('Could not import %s.(%s)\n   Perhaps the content is already installed!' % (file, str(e)) )

            if copy_to_import_dir:
                self._afterProductFileImport(file)

    def _beforeProductFileImport(self, context, productname, filename_list):
        """
        Makes the given file importable by Zope.
        If all goes the right way we're returning true
        """
        
        for filename in filename_list:
            
            product_home = self.getHomeDir()
            import_dir = os.path.join(INSTANCE_HOME, 'import')
            src_file_bin = open(os.path.join(product_home, 'import', filename), 'rb')
            
            # if main import dir does not exists we create it
            if not os.path.exists(import_dir):
                os.makedir(import_dir)

            dest_file_bin = open(os.path.join(import_dir, filename), 'wb')
            dest_file_bin.write(src_file_bin.read())
            src_file_bin.close()
            dest_file_bin.close()

        return True

    def _afterProductFileImport(self, filename):
        """
        Removes all created files created by beforeProductFileImport
        """

        if self._import_dir:
            os.unlink(os.path.join(self._import_dir, filename))
 
    # ***********************************
    # WORKFLOW
    # ***********************************
    
    def assignWorkflowTo(self, portal_type, workflow_name, update=False):
        """
        Assigns the given workflow to the given
        portal_type. If <update> is given, then
        all objects are updated...
        """
        
        wf = getToolByName(self.getPortalObject(), 'portal_workflow')        
        wf.setChainForPortalTypes([portal_type], workflow_name)
        self.msg('Set new workflow %s for objects of type %s' % (workflow_name, portal_type))

        if update:
            wf.updateRoleMappings()
            self.msg('Updated all existing objects with new workflow definition')

    # ***********************************
    # REGISTERING OF MIMETYPES
    # ***********************************
    
    def registerFTPType(self, major, minor, portaltype):
        """
        Register a new content type for FTP/WebDAV Access
        Ex.: audio,mpeg,MP3File
        """

        self.msg('Registering new FTP/WebDAV type: %s/%s -> %s' % (major, minor, portaltype))
        ct = self._root.content_type_registry
        pid = '%s-%s-%s' % (major, minor, portaltype)
        ct.addPredicate(pid, 'major_minor')
        pred = ct.getPredicate(pid)
        pred.edit(major, minor)
        ct.assignTypeName(pid, portaltype)
        ct._p_changed = 1

    # ***********************************
    # GENERIC SETUP
    # ***********************************

    def importExtraGenericContent(self, filename, object, adapterklass, purge=False):
        """ Imports stuff from the given string into the given object. Also pass the
        correct XMLAdapter. """

        configdir = os.path.join(self.getHomeDir(),'profiles')
        filename = os.path.join(configdir, filename)
        f = file(filename)
        content = f.read()
        f.close()
        environ = SetupEnviron(purge=purge)
        environ.site = self._root
        environ._profile_path = configdir
        adapterob = adapterklass(object, environ)
        adapterob.body = content

    def importGenericSetupContent(self, purge=None):
        """ Imports stuff from given directory into portal. You can use the purge
        parameter to indicate that a specific configuration should be emptied
        completely before it is reinstalled. Makes sense for types for catalog configuration.
        Default is True!

        e.g. purge = {'types.xml' : False}
        """

        if purge is None:
            purge = {}

        if purge in [True, False]:
            purge = {'ALL' : purge}

        if HAS_LXML is False:
            self.msg('Could not import profiles w/o lxml!')
            return

        configdir = os.path.join(self.getHomeDir(),'profiles')

        if os.path.exists(os.path.join(configdir, 'rolemap.xml')):
            from Products.GenericSetup.rolemap import importRolemap
            filename = os.path.join(configdir,'rolemap.xml')
            f = file(filename)
            content = f.read()
            f.close()
            environ = SetupEnviron(purge=False)
            environ.site = self._root
            environ._profile_path = configdir
            importRolemap(environ)

        if os.path.exists(os.path.join(configdir, 'actions.xml')):
            self.importConfig(getToolByName(self._root, 'portal_actions'), 
                                os.path.join(configdir,'actions.xml'), purge)

        if os.path.exists(os.path.join(configdir, 'archetype_tool.xml')):
            self.importConfig(getToolByName(self._root, 'archetype_tool'), 
                                os.path.join(configdir,'archetype_tool.xml'), purge)

        if os.path.exists(os.path.join(configdir, 'querytool.xml')):
            self.importConfig(getToolByName(self._root, 'query_tool'), 
                                os.path.join(configdir,'querytool.xml'), purge)

        if os.path.exists(os.path.join(configdir, 'propertiestool.xml')):
            self.importConfig(getToolByName(self._root, 'portal_properties'), 
                                os.path.join(configdir,'propertiestool.xml'), purge)

        if os.path.exists(os.path.join(configdir, 'skins.xml')):
            self.importConfig(getToolByName(self._root, 'portal_skins'), 
                                os.path.join(configdir,'skins.xml'), purge)

        if os.path.exists(os.path.join(configdir, 'workflows.xml')):
            self.importConfig(getToolByName(self._root, 'portal_workflow'), 
                                os.path.join(configdir,'workflows.xml'), purge)
        
            environ = SetupEnviron(profile_path=configdir)
            setuputils.importObjects(self._root.portal_workflow, '', environ)

        if os.path.exists(os.path.join(configdir, 'types.xml')):
            self.importConfig(getToolByName(self._root, 'portal_types'), os.path.join(configdir,'types.xml'), purge)

            # import types from directory types
            self.updateFTI(None, StringIO(), purge)

        if os.path.exists(os.path.join(configdir, 'catalog.xml')):
            self.importConfig(getToolByName(self._root, 'portal_catalog'), os.path.join(configdir,'catalog.xml'), purge)

        if os.path.exists(os.path.join(configdir, 'factorytool.xml')):
            self.importConfig(getToolByName(self._root, 'portal_factory'), os.path.join(configdir,'factorytool.xml'), purge)

        if os.path.exists(os.path.join(configdir, 'membrane_tool.xml')):
            self.importConfig(getToolByName(self._root, 'membrane_tool'), 
                    os.path.join(configdir, 'membrane_tool.xml'), purge)

        if os.path.exists(os.path.join(configdir, 'archetype_tool.xml')):
            self.importConfig(getToolByName(self._root, 'archetype_tool'), 
                    os.path.join(configdir, 'archetype_tool.xml'), purge)

        self.msg('Imported GS content from %s' % configdir)

    def updateFTI(self, productModule, out, purge):
        """
        Update the FTI with the given id from a GenericSetup definition file.
        """
        
        context = self.getPortalObject()
        typesTool = getToolByName(context, 'portal_types')

        # get XML body
        configPath = os.path.join(self.getHomeDir(), 'profiles', 'types')

        for filename in os.listdir(configPath):
            if not filename.endswith('.xml'):
                continue

            definitionName = '%s/%s' % (configPath,filename)
            root = etree.parse(definitionName)
            ftiId = root.xpath('//object/@name')[0]
            obj = getattr(typesTool, ftiId, None)

            # we need to generate a new object because it does not exist
            if obj is None:
                metafrom = root.xpath("//object/property[@name='content_meta_type']")[0].text
                for tinfo in typesTool.listTypeInfo():
                    if tinfo.content_meta_type == metafrom:
                        typesTool.manage_clone(getattr(typesTool, tinfo.id), ftiId)
                        break

                obj = getattr(typesTool, ftiId)

            self.importConfig(obj, definitionName, purge)

    def importConfig(self, obj, filename, purge):
        if not(os.path.exists(filename)):
            raise Exception, "File %s does not exist" % filename

        f = file(filename)
        content = f.read()
        f.close()
        
        # determine purge mode
        if purge.has_key('ALL'):
            purge = purge.get('ALL')
        else:
            purge = purge.get(filename.split('/')[-1], False)

        self.importConfigFromContent(obj, content, purge)
        self.msg('Imported GS config from %s' % filename)
        
    def importConfigFromContent(self, obj, content, purge=True):
        # create import context
        environ = SetupEnviron(purge=purge)

        # apply XML body to adapted obj and environ
        adapted = queryMultiAdapter((obj, environ), IBody)
        if adapted==None:
            raise Exception, "no adapter for %s found" % obj.id
        adapted.body = content

    # ***********************************
    # MULTIPLE LANGUAGES
    # ***********************************
    
    def setupPloneLanguageTool(self, deflang='English', avail=['English']):
        """
        Settings for PloneLanguage Tool
        """

        po = self.getPortalObject()
        if hasattr(po, 'portal_languages'):
            pl = getattr(po, 'portal_languages')
            pl.manage_setLanguageSettings(deflang, avail, setRequestN=0)
            self.msg('Adapted PloneLanguageTool for specified languages')
            
        else: self.msg('Could not find PloneLanguageTool')
        
    # ***********************************
    # PROPERTY EDITING
    # ***********************************

    def installPropertySheet(self, name, object, title=''):
        self.msg('Registering Property Sheet %s in portal_properties' % name)
        pp = self.getPortalProperties()
        if name in pp.objectIds():
            self.msg('Property Sheet already exists - Replacing it.')
            pp.manage_delObjects([name])

        if not object:
            self.error('PropertySheet is empty!!')
            
        else:
            try:
                pp.manage_addPropertySheet(name, title, object)
                self.msg('PropertySheet installed from %s' % str(object.__class__.__name__))
            except:
                self.error('Could not install the property sheet...')
    

    def safeEditProperty(self, obj, key, value, data_type='string', empty_before=False, overwrite=True):
        """ An add or edit function """

        # deleting before adding
        if obj.hasProperty(key) and empty_before:
            obj._delProperty(key)
        
        # updating property
        if obj.hasProperty(key):
            if overwrite:
                obj._updateProperty(key, value)
                self.msg('Changed property %s to %s' % (key, value))

        # new one
        else:
            obj._setProperty(key, value, data_type)
            self.msg('Added property %s' % key)

    def addLinesToProperty(self, obj, key, values):
        if obj.hasProperty(key):
            data = getattr(obj, key)
            if type(data) is TupleType:
                data = list(data)

            # make sure we do not add things twice
            res = []
            for item in values:
                if item not in data:
                    res.append(item)

            values = res

            if type(values) is ListType:
                data.extend(values)

            else:
                data.append(values)
                
            self.msg('Updating property %s (value: %s) of %s' % (key, str(data), str(obj.Title())) )
            obj._updateProperty(key, data)
            
        else:
            self.msg('Adding property %s (value: %s) to %s' % (key, str(values), str(obj.Title())) )
            
            if type(values) is not ListType:
                values = [values]
                
            obj._setProperty(key, values, 'lines')

    def delLinesFromProperty(self, obj, key, values):
        if obj.hasProperty(key):
            data = getattr(obj, key)
            if type(data) is TupleType:
                data = list(data)

            res = []
            for item in data:
                if item not in values:
                    res.append(item)

            obj._delProperty(key)
            obj._setProperty(key, res, 'lines')
            self.msg('Removing lines from property %s: %s' % (key, values))

        else:
            self.error('Could not find property %s on %s scheduled for lines deletion!' % (key, str(obj.Title())) )

