
import random
import Globals
import types

from Acquisition import aq_parent, aq_base, aq_inner
from AccessControl import ClassSecurityInfo
from ZODB.POSException import ConflictError

from Products.Archetypes.utils import shasattr
from Products.Five.browser import BrowserView
from Products.CMFCore.permissions import View
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.CatalogTool import is_folderish
from Products.CMFCore.permissions import setDefaultRoles

ListTypes = [types.ListType, types.TupleType]

# Permission to see management view of Plone
VIEW_CMS_PERMISSION = 'AnthillPublicView: View CMS (Plone interface)'
setDefaultRoles(VIEW_CMS_PERMISSION, ('Manager', ))

PUBLIC_VIEW_TEMPLATE = 'publicview_main'
CMS_VIEW_TEMPLATE = 'plone_main_template'

class PublicView(BrowserView):
    """ Holds utility methods for public view """

    def _getPortalRoot(self):
        purl = getToolByName(self.context, 'portal_url')
        return purl.getPortalObject()

    _getPortalObject = _getPortalRoot

    def _getPortalProperties(self):
        return getToolByName(self.context, 'portal_properties')

    def _getFolderishPortalTypeNames(self):
        types = []
        if hasattr(self._getPortalProperties().site_properties, 'use_folder_tabs'):
            types = self._getPortalProperties().site_properties.use_folder_tabs

        return ['Folder', 'Plone Site', 'Large Plone Folder', 'TempFolder'] + list(types)

    def isSameType(self, t1, t2):
        """ Returns true if both objects
        are of the same type """

        return type(t1) == type(t2)

    def isListType(self, t):
        """ Returns true if object is a list """

        return type(t) in ListTypes   

    def getFolderishContext(self, context):
        """ Returns in all cases a folder. Makes sense when
        you want to be sure that you get a folder (e.g. local_roles).
        Takes also temp object into account and returns
        always the real container. """

        if not is_folderish(context):
            if hasattr(context, 'aq_parent'):
                parent = context.aq_parent
            else:
                if hastattr(aq_base(context), 'im_self'):
                    if hasattr(aq_base(context.im_self), 'context'):
                        return self.getFolderishContext(aq_inner(context.im_self.context))

                raise TypeError, 'Can not determine folder for function'

            return self.getFolderishContext(parent)

        return context

    def getMasterMacro(self, alternate_template = None):
        """ Returns the path to the main view macro. Needed
        to ensure that all slots are defined correctly.
        There are situations where this script returns the view
        template for public view: This happens in situations where security
        management of zope avoids security checks. 
        @see: http://mail.zope.org/pipermail/zope/2001-May/090253.html """

        macropath = 'here/%s/macros/master'
       
        # try to get context to check permission
        # We're looking for the PUBLISHED field
        # and then we extract the folderish parent
        # because of the VIEW_CMS_PERMISSION only living
        # at folderish items
        dact = self
        if hasattr(self, 'request'):
            dact = self.request.get('PUBLISHED', self)
            try:
                dact = self.getFolderishContext(dact)
            except ConflictError:
                raise
            except Exception: 
                raise

            # If using PlonePAS or such we use Portal
            if dact:
                if 'acl_users' in dact.getPhysicalPath():
                    dact = self._getPortalRoot()
            else:
                dact = self._getPortalRoot()

        if self.showPublicView(context=dact):
            return macropath % PUBLIC_VIEW_TEMPLATE
        return macropath % CMS_VIEW_TEMPLATE

    def getMacroPathForLevel(self, level, template='publicmenu', prefix='level_'):
        """ Returns the macro path for the menu level """

        tp = getattr(self._getPortalRoot(), template)
        clevel = level
        while clevel > 0:
            if tp.macros.has_key('%s%s' % (prefix, clevel)):
                return 'here/%s/macros/%s%s' % (template, prefix, clevel)
            else: clevel -= 1

        raise Exception, 'You should at least define one level in %s' % template

    def _cutRootPathFrom(self, path):
        """ Removes root path from given path """
        
        rootlevel = self._getPortalRoot().getPhysicalPath()
        result = []
        for i in range(len(path)):
            __traceback_info__ = (path, rootlevel, i)
            if i <= len(rootlevel)-1:
                if path[i] == rootlevel[i]: continue
            
            result.append(path[i])

        return result

    def objectIsGlobalCurrent(self, objectpath, currentpath):
        """ Returns true if the object given by the
        path is globally selected. In a navtree using
        this method you can mark all items that leads to
        the current item. """

        if not self.isListType(objectpath):
            objectpath = list(objectpath.split('/'))

        if not self.isListType(currentpath):
            currentpath = list(currentpath.split('/'))

        # check if objectpath is subset of currentpath
        if len(objectpath) <= len(currentpath):
            if list(objectpath) == list(currentpath[:len(objectpath)]):
                return True

        return False

    def getMacroPathForLevel(self, level, template='publicmenu', prefix='level_'):
        """ Returns the macro path for the menu level """

        tp = getattr(self._getPortalRoot(), template)
        clevel = level
        while clevel > 0:
            if tp.macros.has_key('%s%s' % (prefix, clevel)):
                return 'here/%s/macros/%s%s' % (template, prefix, clevel)
            else: clevel -= 1

        raise Exception, 'You should at least define one level in %s' % template

    def objectPathIsFolderish(self, path, for_navtree=False):
        """ Returns True if the specified path belongs
        to a folderish object. if the for_navtree parameter
        is given then the navtree_properties.typesNotLinkToFolderContents"""

        if isinstance(path, tuple):
            path = '/'.join(path)

        res = self._getPortalRoot().portal_catalog(path=path, getId=path.split('/')[-1])
        
        # checking for a special navtree parameter
        if for_navtree == True and res:
            props = self._getPortalProperties().navtree_properties
            datypes = getattr(props, 'typesNotLinkToFolderContents', [])
            if res[0].portal_type in datypes:
                return False

        if self.isListType(path):
            path = '/'.join(path)
        
        res2 = []

        # fallback for strange things that can happen.
        # A user can create a folder named x and hereunder
        # a document called x - that yields two brains
        if len(res)>=1:
            res2 = [r for r in res if r.portal_type in self._getFolderishPortalTypeNames() or r.is_folderish]
            res2 = [r for r in res2 if r.getPath()==path] 
      
        if res2:
            return res2[0]
      
        return False

    def publicViewExists(self):
        """ Returns True if a public view can be determined """

        return getattr(self._getPortalRoot(), PUBLIC_VIEW_TEMPLATE, False) and True or False

    def showPublicView(self, viewname=None, context=None):
        """ Returns True if the conditions are in 
        favour of a public view. """

        if not self.publicViewExists():
            return False

        # default rule: show public view if we're in preview mode
        # or current user is an anonymous one
        pms = getToolByName(self._getPortalRoot(), 'portal_membership')
        isanon = pms.isAnonymousUser() 
        can_view = pms.checkPermission(VIEW_CMS_PERMISSION, context and context or self)
        
        # we want to check permission on parent folder
        if context and not can_view:
            # there are some cases where the security engine performs
            # some shortcut for methods that do not require any permission
            # like the default_error_message this leads to anonymous user here
            # see: http://mail.zope.org/pipermail/zope/2001-May/090253.html
            can_view = pms.checkPermission(VIEW_CMS_PERMISSION, self.getFolderishContext(context))

        # dirty hack for the case that isAnonymousUser returns
        # True for auth users. Does not always work
        if isanon:
            if hasattr(self.request, '_auth'):
                if self.request._auth:
                    isanon = False

        return (isanon or not can_view) or self.currentViewIsPreview()

    # handy alias
    inPublicView = showPublicView

    def currentViewIsPreview(self):
        """ Returns True if we're in preview mode """

        # avoid creating a session if there is none
        session = False
        approot = aq_parent(aq_inner(self._getPortalRoot()))
        if hasattr(approot, 'session_data_manager'):
            session = approot.session_data_manager.hasSessionData()

        if session is True:
            if self.request.SESSION.get('plone_preview_mode', CMS_VIEW_TEMPLATE) \
                    != CMS_VIEW_TEMPLATE:
                        return True
        return False
    
    inPreview = currentViewIsPreview
    
    def switchToPublicView(self, context=None, viewname=None):
        """ Switches to public view. If viewname is
        given then switches to the given main template """

        if not self.publicViewExists():
            return

        viewname = viewname or PUBLIC_VIEW_TEMPLATE

        approot = aq_parent(aq_inner(self._getPortalRoot()))
        mng = approot.session_data_manager

        try:
            sess = mng.getSessionData(create=1) 
        except KeyError: 
            try: 
                sess = mng.getSessionData(create=1)
            except KeyError:
                raise Exception, 'Could not allocate Session. Please contact an Administrator.'

        sess['plone_preview_mode'] = viewname

        ct = self._getPortalRoot().absolute_url()
        if context is not None:
            ct = context.absolute_url()

        ref = self.request.get('HTTP_REFERER', None) or ct

        # Remove some special names from url
        props = self._getPortalProperties().site_properties
        if hasattr(props, 'idsNotToSwitch'): 
            names = getattr(props, 'idsNotToSwitch')
        else: names = ['folder_contents', ]
        
        # remove params from ref
        if ref.find('?')>0:
            ref = ref[0:ref.find('?')]
        
        if ref.split('/')[-1] in names:
            ref = '/'.join(ref.split('/')[:-1])

        return self.request.RESPONSE.redirect('%s?cache=%s' % (ref, str(random.random()*10).replace('.', '')))

    def switchToCMSView(self, context=None):
        """ Switches to CMS view """

        if isinstance(context, str):
            context = self.restrictedTraverse(context)
        
        # need to avoid getting empty context
        # because that can lead to strange behaviour
        if context is None:
            context = self._getPortalRoot()

        pms = getToolByName(self, 'portal_membership')
        can_view = pms.checkPermission(VIEW_CMS_PERMISSION, context)
        if can_view:
            self.request.SESSION['plone_preview_mode'] = CMS_VIEW_TEMPLATE
            self.request.SESSION._p_changed = 1

        ct = self._getPortalRoot().absolute_url()
        if context is not None:
            ct = context.absolute_url()

        ref = self.request.get('HTTP_REFERER', None) or ct

        # remove params from ref
        if ref.find('?')>0:
            ref = ref[0:ref.find('?')]
    
        return self.request.RESPONSE.redirect('%s?cache=%s' % (ref, str(random.random()*10).replace('.', '')))

